import 'IReponsitory.dart';
import 'package:hive/hive.dart';

class HiveRepository<T> implements IRepository<T> {
  LazyBox _box;
  HiveRepository() {
    _InitBox();
  }
  void _InitBox<T>() async {
    var keyBox = await Hive.openBox('encryptionKeyBox');
    var key = [];
    if (keyBox.containsKey('key'))
      key = keyBox.get('key');
    else
      key = Hive.generateSecureKey();
    await keyBox.put('key', key);
    _box = await Hive.openLazyBox<T>(T.toString(), encryptionKey: key,
        compactionStrategy: (entries, deletedEntries) {
      return deletedEntries > 5;
    });
  }

  @override
  Future<T> get(dynamic id) async {
    if (this.boxIsClosed) return null;
    return await this._box.get(id);
  }

  @override
  Future<T> add(T object, dynamic key) async {
    if (this.boxIsClosed) return null;
    await this._box.put(key, object);
    return object;
  }

  bool get boxIsClosed => !(this._box?.isOpen ?? false);

  @override
  Future<void> delete(dynamic id) async {
    if (!this.boxIsClosed) this._box.delete(id);
  }

  @override
  Future<void> clear() async {
    if (this.boxIsClosed) this._box.clear();
    this._box.compact();
  }

  @override
  Future<void> update(key, T object) async {
    await add(object, T);
  }
}
