import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:path_provider/path_provider.dart';
import 'AppTranslations.dart';

class Translations {
  final Locale locale;
  Translations(this.locale);
  static Translations of(BuildContext context) =>
      Localizations.of<Translations>(context, Translations);

  static const LocalizationsDelegate<Translations> delegate =
      _LocalizationsDelegate();

  Map<String, String> _localizedStrings;

  Future<bool> _load() async {
    _localizedStrings = Map<String, String>();
    Directory directory = await getApplicationDocumentsDirectory();
    var path =
        '${directory.path}/${AppTranslations().assetLang}${locale.languageCode}.json';
    String jsonString =
        FileSystemEntity.typeSync(path) == FileSystemEntityType.notFound
            ? await _ReadFileRootBundle()
            : await _ReadFileUsingDataUser();
    Map<String, dynamic> jsonMap = json.decode(jsonString);

    _localizedStrings = jsonMap.map((key, value) {
      return MapEntry(key, value.toString());
    });
    return true;
  }

  Future<String> _ReadFileUsingDataUser() async {
    Directory directory = await getApplicationDocumentsDirectory();
    File file = await File(
            '${directory.path}/${AppTranslations().assetLang}${locale.languageCode}.json')
        .create();
    return await file.readAsString();
  }

  Future<String> _ReadFileRootBundle() async {
    return await rootBundle.loadString(
        '${AppTranslations().assetLang}${locale.languageCode}.json');
  }

  UpdateDictionaryLanguage(String content, {Locale localeLang}) async {
    Directory directory = await getApplicationDocumentsDirectory();
    var path =
        '${directory.path}/${AppTranslations().assetLang}${locale.languageCode}.json';
    if (localeLang != null)
      path =
          '${directory.path}/${AppTranslations().assetLang}${localeLang.languageCode}.json';
    File file = await File(path).create(recursive: true);
    return await file.writeAsString(
      '$content',
      mode: FileMode.write,
    );
  }

  String Translate(String key) {
    return _localizedStrings[key] ?? "$key not found";
  }
}

class _LocalizationsDelegate extends LocalizationsDelegate<Translations> {
  const _LocalizationsDelegate();

  @override
  bool isSupported(Locale locale) {
    if (AppTranslations().supportedLanguage.isEmpty)
      AppTranslations().UpdateSupportLanguagesCode();
    return AppTranslations()
        .supportedLanguage
        .any((element) => element.languageCode == locale.languageCode);
  }

  @override
  Future<Translations> load(Locale locale) async {
    Translations localizations = Translations(locale);
    await localizations._load();
    return localizations;
  }

  @override
  bool shouldReload(_LocalizationsDelegate old) => false;
}
